/* Copyright (c) 2009 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT. 
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 * $LastChangedRevision$
 */ 

/** @file
*/

/** @defgroup hal_io hal_io
 * @{
 * @ingroup hal
 * @brief Macros input/output management
*/

#include "hal_platform.h"

#ifndef HAL_IO_H__
#define HAL_IO_H__

#define HAL_IO_P00 0
#define HAL_IO_P01 1
#define HAL_IO_P02 2
#define HAL_IO_P03 3
#define HAL_IO_P04 4
#define HAL_IO_P05 5
#define HAL_IO_P06 6
#define HAL_IO_P07 7

#define HAL_IO_P08 8
#define HAL_IO_P09 9
#define HAL_IO_P10 10
#define HAL_IO_P11 11
#define HAL_IO_P12 12
#define HAL_IO_P13 13
#define HAL_IO_P14 14
#define HAL_IO_P15 15

#define HAL_IO_P16 16
#define HAL_IO_P17 17
#define HAL_IO_P18 18
#define HAL_IO_P19 19
#define HAL_IO_P20 20
#define HAL_IO_P21 21
#define HAL_IO_P22 22
#define HAL_IO_P23 23

#define HAL_IO_P24 24
#define HAL_IO_P25 25
#define HAL_IO_P26 26
#define HAL_IO_P27 27
#define HAL_IO_P28 28
#define HAL_IO_P29 29
#define HAL_IO_P30 30
#define HAL_IO_P31 31

#define HAL_IO_INPUT_DIR |= 
#define HAL_IO_OUTPUT_DIR &= ~

#define HAL_IO_OUTPUT 0
#define HAL_IO_INPUT 1

#define HAL_IO_OUTPUT_NORMAL_STRENGTH 0x00
#define HAL_IO_OUTPUT_HIGH_STRENGTH 0x60
#define HAL_IO_INPUT_BUF_ON_NO_PULL 0x00
#define HAL_IO_INPUT_BUF_ON_PULL_DOWN 0x20
#define HAL_IO_INPUT_BUF_ON_PULL_UP 0x40
#define HAL_IO_INPUT_BUF_OFF 0x60

/**@brief Macro to configure an I/O.
 *  @details
 *  This macro configures a given I/O to input or output with pullup/buffer configuration.
 *  @param io_name I/O to configure.
 *  @param is_input Indicate if the I/O is to be set as an input.
 *  @param io_mode Pull resistor and buffer configuration (must be HAL_IO_OUTPUT_NORMAL_STRENGTH, HAL_IO_OUTPUT_HIGH_STRENGTH, HAL_IO_INPUT_BUF_ON_NO_PULL,
 *  HAL_IO_INPUT_BUF_ON_PULL_DOWN, HAL_IO_INPUT_BUF_ON_PULL_UP, or HAL_IO_INPUT_BUF_OFF).
*/
#define HAL_IO_CONFIG(io_name, is_input, io_mode) ((is_input == 1) ? (LPC_GPIO0->FIODIR = (LPC_GPIO0->FIODIR & ~(1UL << io_name))) : (LPC_GPIO0->FIODIR |= (1UL << io_name)))

/**@brief Macro to set an output.
 *  @details
 *  This macro sets the given output to the given level (1 -> high, 0 -> low).
 *  @param io_name Output to change.
 *  @param io_state Level to set.
*/
#define HAL_IO_SET_STATE(io_name, io_state) ((io_state == 1) ? (LPC_GPIO0->FIOSET = (1UL << io_name)) : (LPC_GPIO0->FIOCLR = (1UL << io_name)))

/**@brief Macro that reads the current state of an input.
 *  @details
 *  This macro reads the current state of a logical input.
 *  @param io_name Input to read.
 *  @return Input state level (1 if high, 0 if low).
*/
#define HAL_IO_READ(io_name) ((LPC_GPIO0->FIOPIN & (1UL << io_name)) == (1UL << io_name))


#endif //HAL_IO_H__

/** @} */
