/* Copyright (c) 2009 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 * $LastChangedRevision$
 */


#include "hal_platform.h"
#include "lib_aci.h"
#include "lib_traces.h"
#include "system.h"
#include "services.h"
#include <string.h>

#ifdef SERVICES_PIPE_TYPE_MAPPING_CONTENT
    static KEYWORD_CODE services_pipe_type_mapping_t
        services_pipe_type_mapping[NUMBER_OF_PIPES] = SERVICES_PIPE_TYPE_MAPPING_CONTENT;
#else
    #define NUMBER_OF_PIPES 0
    static services_pipe_type_mapping_t * services_pipe_type_mapping = NULL;
#endif

static KEYWORD_CODE hal_aci_data_t
    setup_msgs[NB_SETUP_MESSAGES] = SETUP_MESSAGES_CONTENT;

#define ACI_CMD_Q_SIZE 16

static uint8_t data_credit_available;
static hal_aci_data_t aci_data;
static hal_aci_data_t aci_cmd;

void SystemInit(void)
{

}

aci_status_code_t aci_setup(uint8_t num_cmds, uint8_t num_cmd_offset)
{
  uint8_t i = 0;
  uint8_t evt_count = 0;
  aci_evt_t * aci_evt;

  while (i < num_cmds)
  {
    //Copy the setup ACI message from Flash to RAM
    //Add 2 bytes to the length byte for status byte, length for the total number of bytes
    memcpy(&aci_cmd, &(setup_msgs[num_cmd_offset+i]), (setup_msgs[num_cmd_offset+i].buffer[0])+2);

    //Put the Setup ACI message in the command queue
    if (!hal_aci_tl_send(&aci_cmd))
    {
      return ACI_STATUS_ERROR_INTERNAL;
    }

    i++;
  }
  while (1)
  {
    //We will sit here if we do not get the same number of command response evts as the commands sent to the ACI
    if (true == lib_aci_event_get(&aci_data))
    {
      aci_evt = (aci_evt_t *)&(aci_data.buffer[0]);
      evt_count++;

      if (ACI_EVT_CMD_RSP != aci_evt->evt_opcode)
      {
        //Got something other than a command response evt -> Error
        return ACI_STATUS_ERROR_INTERNAL;
      }
      if (num_cmds == evt_count)
      {
        break;
      }
    }
  }

  return ((aci_status_code_t)aci_evt->params.cmd_rsp.cmd_status);
}

void main_application_loop();

int main(void)
{
	lib_aci_init(services_pipe_type_mapping, NUMBER_OF_PIPES, setup_msgs, NB_SETUP_MESSAGES);

	for (;;)
	{
	  main_application_loop();
	}
}

void main_application_loop(void)
{
  aci_evt_t * aci_evt;

  // We enter the if statement only when there is a ACI event available to be processed
  if (lib_aci_event_get(&aci_data))
  {
    aci_evt = (aci_evt_t *)&(aci_data.buffer[0]);

    //@todo change this so that the commands and events can be processed here in a switch case instead of callbacks
    switch(aci_evt->evt_opcode)
    {
        case ACI_EVT_DEVICE_STARTED:
        {
          data_credit_available = aci_evt->params.device_started.credit_available;
          switch(aci_evt->params.device_started.device_mode)
          {
            case ACI_DEVICE_SETUP:
            {
              uint8_t i=0;

              if(ACI_CMD_Q_SIZE >= NB_SETUP_MESSAGES)
              {
                aci_setup(NB_SETUP_MESSAGES, 0);
              }
              else
              {
                for(i=0; i<(NB_SETUP_MESSAGES/ACI_CMD_Q_SIZE); i++)
                {
                  aci_setup(ACI_CMD_Q_SIZE, 0+(ACI_CMD_Q_SIZE*i));
                }
                if ((NB_SETUP_MESSAGES % ACI_CMD_Q_SIZE) != 0)
                {
                 aci_setup(NB_SETUP_MESSAGES % ACI_CMD_Q_SIZE, (ACI_CMD_Q_SIZE*i));
                }
              }
            }
            break;

          case ACI_DEVICE_STANDBY:
            lib_aci_connect(180/* in seconds */, 0x0100 /* advertising interval 100ms*/);
            break;
          }
        }
        break; //ACI Device Started Event
      case ACI_EVT_CMD_RSP:
        //If an ACI command response event comes with an error -> stop
        if (ACI_STATUS_SUCCESS != aci_evt->params.cmd_rsp.cmd_status)
        {
          //ACI ReadDynamicData and ACI WriteDynamicData will have status codes of
          //TRANSACTION_CONTINUE and TRANSACTION_COMPLETE
          //all other ACI commands will have status code of ACI_STATUS_SCUCCESS for a successful command
          while (1);
        }
        break;
      default:

        break;
    }
  }
  else
  {
    // No event in the ACI Event queue
    // Arduino can go to sleep now
  }
}
